@@ -1,4 +1,4 @@
-/* $Id: aggregate.c,v 1.8 2001/05/17 13:52:01 jabley Exp $
+/* $Id: aggregate.c,v 1.10 2001/11/01 17:02:50 jabley Exp $
  * 
  * Copyright (c) 2000-2001 by Metromedia Fiber Network Services, Inc.
  * 
@@ -35,6 +35,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
+#include <ctype.h>
 
 #include <unistd.h>
 #include <sys/types.h>
@@ -52,10 +53,6 @@
      p->masklen < q->masklen) )
 
 
-int inet_aton(const char *cp, struct in_addr *pin);
-
-
-
 struct filter_list
 {
   struct in_addr prefix;
@@ -81,17 +78,96 @@ int moanf(int fatal, char *fmt, ...)
 }
 
 
-#ifndef HAVE_INET_ATON
-#ifdef HAVE_INET_PTON
-int
-inet_aton(const char *cp, struct in_addr *pin)
+/* we are rolling our own inet_aton() variant here, since the
+ * common libc practice of interpreting leading zeros on octets
+ * as octal numbers is non-intuitive, and is not compatible
+ * with zero-padded (decimal) data from whois.radb.net. Thanks
+ * to Arnold Nipper for noticing. This function was mainly
+ * ripped from OpenBSD source, since that's what happened to be
+ * handy.
+ */
+
+int my_inet_aton(const char *cp, struct in_addr *addr)
 {
-   return inet_pton(AF_INET, cp, pin);
+  in_addr_t val;
+  int n;
+  char c;
+  u_int parts[4];
+  u_int *pp = parts;
+
+  c = *cp;
+  for (;;) {
+    /*
+     * collect number up to ``.'' Values are not specified as for C;
+     * preceding 0s still cause the digits to be interpreted as
+     * decimal, and "x" characters are illegal. This is different
+     * to the behaviour of the libc routine inet_aton().
+     */
+    if (!isdigit(c))
+      return (0);
+
+    val = 0;
+
+    for (;;) {
+      if (isdigit(c)) {
+        val = (val * 10) + (c - '0');
+        c = *++cp;
+      } else
+        break;
+    }
+
+    if (c == '.') {
+      /*
+       * Internet format:
+       *      a.b.c.d
+       *      a.b.c   (with c treated as 16 bits)
+       *      a.b     (with b treated as 24 bits)
+       */
+      if (pp >= parts + 3)
+        return (0);
+      *pp++ = val;
+      c = *++cp;
+    } else
+      break;
+  }
+
+  /* Check for trailing characters. */
+  if (c != '\0' && (!isascii(c) || !isspace(c)))
+    return (0);
+
+  /* Concoct the address according to the number of parts specified. */
+  n = pp - parts + 1;
+  switch (n) {
+    case 0:       /* initial nondigit */
+      return (0);
+
+    case 1:       /* a -- 32 bits */
+      break;
+
+    case 2:       /* a.b -- 8.24 bits */
+      if ((val > 0xffffff) || (parts[0] > 0xff))
+        return (0);
+      val |= parts[0] << 24;
+      break;
+
+    case 3:       /* a.b.c -- 8.8.16 bits */
+      if ((val > 0xffff) || (parts[0] > 0xff) || (parts[1] > 0xff))
+        return (0);
+      val |= (parts[0] << 24) | (parts[1] << 16);
+      break;
+
+    case 4:       /* a.b.c.d -- 8.8.8.8 bits */
+      if ((val > 0xff) || (parts[0] > 0xff) || (parts[1] > 0xff) || \
+          (parts[2] > 0xff))
+        return (0);
+      val |= (parts[0] << 24) | (parts[1] << 16) | (parts[2] << 8);
+      break;
+  }
+
+  if (addr)
+    addr->s_addr = htonl(val);
+  return (1);
 }
-#else /* ! HAVE_INET_PTON */
-#error "inet_aton or inet_pton required"
-#endif /* ! HAVE_INET_PTON */
-#endif /* ! HAVE_INET_ATON */
 
 
 int main(int argc, char **argv)
@@ -195,7 +271,7 @@ int main(int argc, char **argv)
       }
     }
 
-    if (inet_aton(buf, &prefix) == 0)
+    if (my_inet_aton(buf, &prefix) == 0)
     {
       moanf(0, "[line %d] can't parse prefix '%s'; ignoring line", line, \
         buf);
